#!/usr/bin/perl
use strict       ;
use File::Copy   ;
use Tie::File    ;
use Fcntl        ;
use Term::Cap ;
use Term::ANSIColor qw(:constants);
use Getopt::Std  ;

# updated on 13th febrauary, 2009 by patrick kck
# updated on 2nd april, 2009 by patrick kck
# updated on 15th june, 2009 by patrick kck
# updated on 26th july, 2009 by patrick kck
# updated on 9th february, 2011 by patrick kck
# updated on 8th august, 2011 by patrick kck (update of the help menus)

my @cut_positions       = () ;
my $answer_remain_stems = () ;
my $answer_opening      = () ;
my $structurestring     = () ;
my $file_struc          = () ;
my $length_seq          = () ;

&argv_handling ;
&menu          ;

sub argv_handling{
	
	
	my ( $commandline ) = join "", @ARGV ;
	
	if ( $commandline ne undef ){ 
		
		$commandline =~ s/ |\s+// ;
		my @commands = split "-", $commandline ;
		
		REPEAT_ARGV:
		for my $single_command ( sort @commands ){
			
			$single_command =~ /^r$/i and do { &remain_stems } ;
			$single_command =~ /^p$/i and do { &preface } ;
			$single_command =~ /^h$/i and do { &help } ;
			$single_command =~ /^s$/i and do { &start } ;
		}
	}
}

sub help{    system('cls') ;
	
	print
 <<info;
 
	General Information and Usage:
	-------------------------------
	ALICUT V1.0 removes all ALISCORE identified RSS positions 
	in given FASTA file(s) which are listed in the FASTA file cor-
	responding "List" file(s) of ALISCORE.
	
	To start ALICUT via menu type:
	perl ALICUT_v1.0.pl <enter>
	To start ALICUT directly via command line type:
	perl ALICUT_v1.0.pl -s <enter>
	
	To run ALICUT, the ALISCORE inputfile(s) and "List" outputfile(s)
	have to be placed in the same folder as ALICUT. The ALISCORE 
	"List" outfile(s) must contain the ALISCORE identified
	RSS positions in one single line, separated by whitespace.
	
	If structure sequences are implemented, ALICUT automatically 
	replace brackets of non RSS positions by dots if they are paired 
	with RSS identified positions. To remain all stem positions despite
	identified RSS within stem positions 
	type via menu:
	<r> <enter>
	
	via commandline:
	perl ALICUT_v1.0.pl -r -s <enter>
	
	ALICUT V1.0 can handle unlimited FASTA files in one single run.
	The sole condition is that no filename, neither the ALISCORE FASTA
	inputnames(s) nor the ALISCORE "List" outfile(s) are changed
	since the usage of ALISCORE! 
	
	ALICUT reads IN the ALISCORE FASTA infile(s) and "List" outfile(s), 
	excludes the ALISCORE identified RSS positions listed in the 
	"List" file(s) and saves the restricted sequences as a new FASTA 
	outfile. 
	
	If two "List" files are generated from an equal named FASTA file 
	(e.g. the first "List" file includes ALSICORE identified RSS along 
	a NJ tree while the second "List" file includes identified RSS 
	positions identified by all single comparisons) the first ALICUT 
	outfile will be overwritten by the second one.
	
	ALICUT V1.0 generates an info file "ALICUT_info" within 
	the new folder "/ALICUT_output". In this file information about
	number and percent of removed positions is given for each restricted
	FASTA file. If one ALICUT outfile was overwritten by a second one, 
	it can be seen in the info file which one. 
	
	If a structure sequence is given within ALISCORE FASTA infile(s), 
	ALICUT generates a structure info file for each FASTA file which 
	lists corresponding stem positions, loop positions and the percentage
	of both structure elements.
	
info
;

	print  "\tBACK to ALICUT MAIN-Menu:\t\t type <return>\n"                    ;
	print  "\n\t------------------------------------------------------------\n\t"  ;

	chomp ( my $answer_xy = <STDIN> );

	&menu ;
	
}

sub error1{  system('cls') ;
	
	print
 <<error1;

	Error1-Reports:
	-------------------------------
	Error1 reports are depending on potential wrong "List" 
	files. ALICUT checks each "List" file before it starts and 
	if something seems not ok, ALICUT will print a small report 
	on the desktop:
	
	>"Can not find original named ALISCORE listfile in .txt format!"
	A reason for this error message can be that the ALSICORE "List" 
	outfile is in .txt format, but the file extension is missing. 
	Another reason can be that ALSICORE the "List" outfile 
	is not in the same folder as ALICUT.
	
	>"File is not a original named ALISCORE listfile!"
	This means that the textfile is not a original ALISCORE "List" 
	outfile. Maybe the filename has changed or the file is just 
	another textfile which can be denied. In latter case the error 
	message can be ignored. Otherwise change the filename back to 
	its original ALISCORE outfile name. To find the "List" outfile" 
	of ALISCORE. ALICUT orientates itselfs by the word "List" which 
	must be implemented in the ALISCORE "List" outfile. 
	Upper or lower cases are ignored.

	>"File has no ALISCORE list format!"
	The format of the ALISCORE "List" outfile is not correct. 
	Only integers separated by one whitespace in the first line
	are allowed. The first line must start and end with an integer
	not with a whitespace. 

error1
;
	print  "\tBACK to ALICUT MAIN-Menu:\t\t type <return>\n"                    ;
	print  "\n\t------------------------------------------------------------\n\t"  ;

	chomp ( my $answer_xy = <STDIN> );

	&menu ;
}

sub error2{  system('cls') ;
	
	print
 <<error2;

	Error2-Reports:
	-------------------------------
	Error2 reports depend most likely on the given 
	ALISCORE FASTA inputfile(s). 
	
	>"Can not find FASTA file!"
	Maybe the name of the given "List" file does not agree with the 
	associated FASTA file. Normally, if nothing was changed from the 
	ALISCORE output, the FASTA filename is part of the "List" file-
	name: <FASTAfile.fas_List...txt>. Alternatively the FASTA file
	is not within the same folder.
	
	>"FASTA file is empty!"
	Well, what do I have to say about this error?!
	
	>"Taxon name is missing in FASTA file!"
	ALICUT can not find a sequence corresponding taxonname within 
	the ALISCORE FASTA infile.
	
	>"Wrong sequence signs in sequence of FASTA file!"
	ALICUT scans every FASTA sequence for forbidden signs, 
	e.g. "=", "2". Only nucleotide states, aminoacid states, gaps (-),
	missing data (?) and ambiguity characters are allowed.
	
	>"No equal sequence lengths in FASTA file!"
	ALICUT checks the ALISCORE FASTA infile(s) for equal sequence length. 
	If sequences have no equal length, ALICUT stops with this error 
	message.
	
	>"Too short Sequence lengths in FASTA file!"
	If the sequences were to short to cut all ALISCORE identified RSS 
	positions ALICUT stops with this error message. 
	Maybe the given "List" file does not depent to the given 
	FASTA file.
	
	>"Multiple taxa in FASTA file"
	ALICUT does not restrict files with identic sequence names within 
	an ALISCORE FASTA infile.
	
	>"Sequence missing in FASTA file!"
	ALICUT can not find a taxonname corresponding sequence in the 
	ALISCORE FASTA infile.
	
error2
;

	print  "\tBACK to ALICUT MAIN-Menu:\t\t type <return>\n"                    ;
	print  "\n\t------------------------------------------------------------\n\t"  ;

	chomp ( my $answer_xy = <STDIN> );

	&menu ;
}

sub preface{ system('cls') ;

print
<<preface
	
	--------------------FASconCAT PREFACE---------------------
	
	Version     : 1.0
	Language    : PERL
	Last Update : 8th August, 2011
	Author      : Patrick Kck, ZFMK Bonn GERMANY
	e-mail      : ali_score\@web.de
	Homepage    : http://www.zfmk.de
	
	This program is free software; you can whitedistribute it 
	and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation ; 
	either version 2 of the License, or (at your option) any 
	later version.

	This program is distributed in the hope that it will be 
	useful, but WITHOUT ANY WARRANTY; without even the 
	implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for 
	more details. 

	You should have received a copy of the GNU General Public 
	License along with this program; if not, write to the Free 
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, 
	USA.
	
	For further free downloadable programs visit:
	www.zfmk.de/
	
	------------------------------------------------------------

preface
; 

	print  "\tBACK to ALICUT MAIN-Menu:\t\t type <return>\n"                       ;
	print  "\n\t------------------------------------------------------------\n\t"  ;

	chomp ( my $answer_xy = <STDIN> );

	&menu;
}

sub remain_stems{ 

	$answer_remain_stems = "yes" ;

print
<<structure_info;

	RSS identified stem positions are remained !
	
structure_info
;
	&menu;
}


sub menu{    system("cls") ;
	
	printf "\n%68s\n", "------------------------------------------------------------"     ;
	printf "%49s\n"  , "Welcome to ALICUT V1.0 !"                                         ;
	printf "%60s\n"  , "a Perlscript to cut ALISCORE identified RSS"                      ;
	printf "%57s\n"  , "written by Patrick Kueck (ZFMK, Bonn)"                            ;
	printf "%68s\n\n", "------------------------------------------------------------"     ;
	
	print "\n\tSTART ALICUT:\t\ttype <s> <return>"                                        ;
	print "\n\tQUIT  ALICUT:\t\ttype <q> <return>"                                        ;
	print "\n\tREMAIN STEMS:\t\ttype <r> <return>"                                        ;
	print "\n\tHELP:\t\t\ttype <h> <return>"                                              ;
	print "\n\tPREFACE:\t\ttype <p> <return>"                                             ;
	print "\n\tERROR REPORT:\t\ttype <X> <RETURN>\n"                                      ;
	print "\n\t(X = Error associated number: 1 or 2)"                                     ;
	
	printf "\n%68s\n\t", "------------------------------------------------------------"   ;

	my       $answer_opening =  &commandline ;
	
	until  ( $answer_opening =~ /^s$|^r$|^p$|^h$|^1$|^2$|^q$/i ){ 
		
		my @errorreport = "\n\t!COMMAND-ERROR!: unknown command \"$answer_opening\"!\n" ;
		$answer_opening =  &commandline(@errorreport) ;
	}

	$answer_opening =~ /^s$/i      and do { &start }   ;
	$answer_opening =~ /^r$/i      and do { &remain_stems } ;
	$answer_opening =~ /^q$/i      and do { exit }     ;
	$answer_opening =~ /^h$/i      and do { &help }    ;
	$answer_opening =~ /^1$/       and do { &error1 }  ;
	$answer_opening =~ /^2$/       and do { &error2 }  ;
	$answer_opening =~ /^p$/i      and do { &preface }
	
}

sub start{

	my @fasta_files     = () ;
	my $structurestring = () ;
	my $j               = 0  ;
	
	
	open  OUTinfo, ">>ALICUT_info.xls" ;
	print OUTinfo  "\nUsed List File\tUsed Fasta file\tbp before\tbp after\tremaining bp [%]\n"  ;
	
	# Read IN of all .txt files within the same folder as ALICUT and handle it
	READING:
	foreach my $file ( <*.txt> ) {
		
		# Set counter +1
		$j++;
		
		# Define local variables for each read in of .txt file
		my ( %seen, @inputfile, @zahlenreihe, @fasta_cut ) = ();
		
		
		# Evaluation of each .txt file for ALISCORE-list outfile
		if  ( $file =~ /List/i ) { open IN, "<$file" or die "Can not find original named ALISCORE listfile in .txt format" }
		else                     { warn "\tError1: $file is not a original named ALISCORE listfile!\n"        ; next READING }
		
		
		# Read in of the ALISCORE-list outfile
		chomp  ( my $line = <IN> ) ;
		
		unless ( $line =~ /^(\d+ )+\d+$|^\d+$|^$/ ) { warn "\tError1: $file has no ALISCORE list format\n" ; next READING }
		#if     ( $line =~ /^$/ ) { goto EMPTY_LISTFILE }
		
		# Total number of randomized identified positions
		my $number_cut_positions     = ( @cut_positions = split " ", $line ) ;
		
		
		# For opening the right fastafile for each listfile the last signs of the listfile must be removed from "filename.fas_List_random.txt" to "filename.fas"
		my $wo                       = index  ( $file, "List" )          ;
		my $file_fasta               = substr ( $file, 0, ( $wo - 1 ) )  ;
		   $file_struc               = substr ( $file, 0, ( $wo - 4 ) )  ;
		
		
		#checking line feeds and making shure that just \n remains
		(open IN , $file_fasta ) or warn "\tError2: $file_fasta can not be opened!\n" and next READING ;
		
		(tie ( my @data, 'Tie::File', $file_fasta )) ;
		
		warn "\tError2: $file_fasta is empty!\n" and next READING if 0 == @data ;
		
		map { s/\r\n/\n/g } @data ;
		map { s/\r/\n/g   } @data ;
		
		untie @data ;
		
		
		# Read in of the original ALISCORE fasta infile which belongs to the listfile
		open INfas, "<$file_fasta" or warn "\tError2: Can not find $file_fasta!\n" and next READING ;
		
		
		# Read IN the list associated FASTA file to array @inputfile
		chomp ( @inputfile = <INfas> ) ;
		warn "\tError2: File $file_fasta is empty!\n" if 0 == @inputfile and next READING ;
		
		
		# Handle the FASTA file in the way that sequencename and sequence alternate in each line
		@inputfile                   = fas_bearbeiten ( @inputfile ) ;
		my $number_characters_before = $inputfile[1] ;
		
		
		# Generate a hash: key=>taxon, value => sequenz
		my %sequence                 = @inputfile ;
		my @values                   = values %sequence ;
		
		
		# Determine basepositions before und after cut. Output of cuttings as total number and in percent
	       $length_seq               = length $values[0] ;
		my $number_characters_after  = $length_seq-$number_cut_positions ;
		
		
		# Evaluation of sequences
		for my $key ( keys %sequence ){
			
			
			# if whitespace are between ">" and the next sign within sequence name, delete these whitespaces
			$key =~ s/^\>\s*/\>/g ;
			
			# if whitespaces between last sign and newline in sequence name, delete these whitespaces
			$key =~ s/\s*$//g ;
			
			my %FASTA = () ;
			
			warn "\tError2: Taxon name missing in $file_fasta!\n"                                 and next READING if $key =~ /^\>$/                                                                ;
			warn "\tError2: Sequence missing in $file_fasta!\n"                                   and next READING if $sequence{$key} =~ /^\n$|^$/                                                  ;
			warn "\tError2: Multiple taxa of $key in $file_fasta\n"                               and next READING if defined $FASTA{$key}                                                          ;
			warn "\tError2: $key in $file_fasta is not in FASTA format\n"                         and next READING if $key !~ /^\>/                                                                 ;
			warn "\tError2: Sequence of $file_fasta involves forbidden signs in $key\n"           and next READING unless $sequence{$key} =~ /A|C|G|T|U|-|N|Y|X|R|W|S|K|M|D|V|H|B|Q|E|I|L|F|P|\?/i  ;
			warn "\tError2: Sequences of $file_fasta have no equal length\n"                      and next READING if length $sequence{$key} != $length_seq                                         ;             
			warn "\tError2: Sequence lengths in $file_fasta are to short to cut all positions\n"  and next READING if $length_seq < $cut_positions[ $#cut_positions ]                               ;
			
			# Structure tagging, handling and dot allocation
			if ( $sequence{$key} =~ /.*\(.*\).*/ ){
				
				$structurestring =  $sequence{$key} ; 
				$structurestring =~ s/-/./g ;
				$sequence{$key}  =  &structure_handling ( $structurestring ) ;
			}
		}
		
		my $percent_left = sprintf "%.1f", ( $number_characters_after / $length_seq ) * 100 ;
		$percent_left =~ s/\./,/g ;
		
		
		# Generate a array of numbers from 1 till last position to cut
		for ( 0..($length_seq-1), my $i=0 ) { push @zahlenreihe, $i; $i++ }
		
		
		# The ALISCORE-list file numbered the first seqposition with 1, the first position in perl is numbered 0 !
		# Therefore every position of the list-file must be substrated with -1 to get the identical positions !
		for my $value ( @cut_positions ) { $value = $value - 1 }
		
		
		# Read IN of cut positions to a hash
		for ( @cut_positions ){ $seen{$_} = 1 }
		
		
		# Compare array of numbers with @cut_positions
		for ( @zahlenreihe ){ unless ( $seen{$_} ){ push @fasta_cut, $_ } }
		
		
		open OUT, ">ALICUT_$file_fasta" ;
		
		
		# Assume uncut positions to $final and print them out to ALICUT_$file_fasta
		for ( keys %sequence ){
			
			my @bases = split "", $sequence{$_}          ;
			my @final = map { $bases[$_] } @fasta_cut    ;
			my $final = $_."\n".( join "", @final )."\n" ;
			
			print OUT "$final" ;
		}
		close OUT;
		
		# Readout of extra infos to ALICUT_info
		print OUTinfo  "$file\t$file_fasta\t$length_seq\t$number_characters_after\t$percent_left\n" ;
		print          "\tDone  : $file cut to ALICUT_$file_fasta\n" ;
		
		push @fasta_files, $file_fasta ;
		
	}
	
	
	# Count the number of right handled FASTA files
	my $N_bearbeitet = @fasta_files ;
	
	close IN       ;
	close INfas    ;
	close OUTinfo  ;
	
	# Print OUT number of right handled FASTA files in relation to total number of files
	printf "\n%68s\n",   "------------------------------------------------------------" ;
	printf "%42s\n",     "$N_bearbeitet FASTA file(s) correctly handled!"               ;
	printf "%57s\n",     "Further infos are printed out in Alicut_info.txt!"            ;
	printf "\n%63s\n",   "ALICUT V1.0 Finished! Thank you for using it! Good bye!"      ;
	printf "%68s\n",     "------------------------------------------------------------" ;
	
	
	# set timer
	my ( $user, $system, $cuser, $csystem ) = times ;
	
print <<TIME;

			***  time used: $user sec  ***

TIME
#

	exit ;
}

sub fas_bearbeiten{
	
	my @infile = @_                   ;
	
	grep  s/(\>.*)/$1\t/,     @infile ;
	grep  s/ //g,             @infile ;
	grep  s/\n//g,            @infile ;
	grep  s/\t/\n/g,          @infile ;
	grep  s/\>/\n\>/g,        @infile ;
	my $string = join "",     @infile ;
	@infile    = split "\n",  $string ;
	shift                     @infile ;
	return                    @infile ;
}

sub structure_handling{

	my @pair_infos            =  ()                           ;
	my @forward               =  ()                           ;
	my @structurestring       =  ()                           ;
	my @loops                 =  ()                           ;
	my @pairs                 =  ()                           ;
	my %structure_of_position =  ()                           ;
	my %seen_struc            =  ()                           ;
	my @structures            =  split "", $structurestring   ;
	
	
	# Stem assignment
	my  $i = 0                                                                                                         	                  ;
	CHECKING:
	for ( @structures ){ $i++                                                                                                             ;
		
		SWITCH:
		$structure_of_position{$i} = $_                                                                                                   ;
		
		if ( $_  =~ /\(/ ){ push @forward, $i                                                                          and next CHECKING  }
		if ( $_  =~ /\)/ ){ my $pair_1 = pop @forward; push @pairs, ( $pair_1, $i ); push @pair_infos, ( $pair_1.":".$i ); next CHECKING  }
		if ( $_  =~ /\./ ){ push @loops,   $i                                                                          and next CHECKING  }
	}
	
	@pair_infos  =  reverse @pair_infos                                                                                                   ;
	
	
	
	
	# Generate listfiles for structure_info file
	my $pairlist =  join "\n\t\t\t\t\t", @pair_infos   ;
	my $looplist =  join "\n\t\t\t\t\t", @loops        ;
	
	
	# Number and proportion of stem and loop positions for structure info file
	my $N_total  =  @structures                        ;
	my $N_stems  =  @pair_infos                        ;
	my $N_loops  =  $N_total - ( $N_stems * 2 )        ;
	my $P_loops  =  ( $N_loops / $N_total ) * 100      ;
	my $P_stems  =  100 - $P_loops                     ;

	
	# Open structure info outfile
	open OUTstruc, ">ALICUT_Struc_info_${file_struc}txt"                             ;
	
	# Print out
	print OUTstruc "\nOriginal structure information identified in $file_struc:\n\n"  ;
	print OUTstruc "- Number of characters:\t\t\t$N_total\n"                          ;
	print OUTstruc "- Number of single loop characters:\t$N_loops [$P_stems %]\n"     ;
	print OUTstruc "- Number of paired stem characters:\t$N_stems [$P_loops %]\n"     ;
	print OUTstruc "\n- Paired stem positions:\t\t$pairlist\n\n"                      ;
	print OUTstruc "\n- Loop positions:\t\t\t$looplist\n"                             ;

	close OUTstruc;
	
	if  ( $answer_remain_stems =~ /yes/i ){
		
		my @cut_positions2 = ();
		
		# Remain rss identified stem positions within the MSA
		for ( @pairs ){ $seen_struc{$_} = 1                                           }
		for ( @cut_positions ){ unless ( $seen_struc{$_} ){ push @cut_positions2, $_  } }
		@cut_positions = @cut_positions2                                              ;
	}
	
	else{
		
		my %pair = @pairs;
		
		# Replace paired structure positions of rss identified positions by dots
		for my $bp_for ( keys %pair ){
			
			for my $rss ( @cut_positions ){
				
				if ( $bp_for        == $rss ){ $structure_of_position{$pair{$bp_for}}  = "." ; last }
				if ( $pair{$bp_for} == $rss ){ $structure_of_position{$bp_for}         = "." ; last }
			}
		}
	}
	
	for    ( my $k=1; $k<=$length_seq; $k++ ){ push @structurestring, $structure_of_position{$k}   }
	my     $structure_string_neu = join "", @structurestring                                       ;
	return $structure_string_neu                                                                   ;
	
}

sub commandline{

	my ($errorreport) = @_ ;
	print  "\n\t$errorreport";
	print  "\n\tCOMMAND:\t "                                                          ;
	
	chomp ( my $sub_answer_opening = <STDIN> );

	print  "\n\t------------------------------------------------------------\n"        ;
	
	return $sub_answer_opening;
}	



















































