#!usr/bin/perl
# written by patrick kck 2008
# updated by patrick kck 2011

=head1 General Usage

    ``ESTa.pl'' is written and downloads the actual EST-summary list from the NCBI-Genbank (\url{http://www.ncbi.nlm.nih.gov/dbEST/dbEST\_summary.html}) for taxa with more than 1.000 EST database entries. The actual list will be stored in a EST database list file (``EST-request\_(NCBI).txt''). Database entries of an earlier downloaed database list file are replaced if an earlier version and the ESTa perlscript are placed in the same folder. In addition to the ``EST-request\_(NCBI).txt'' file, new database entries are printed out in a separate text file ``New\_EST\_entries.txt'', too.
	
	To start ESTa open the terminal window of your running system, navigate through your directory path to the folder where ESTa is located, and execute ESTa by typing the name of the ESTa version followed by <enter>. Be sure that an internet connection exist.

  \begin{itemize}
    \item C:\verb|\|ESTa\_Folder$>$ perl ESTa\_v0.1.beta.pl $<$enter$>$ $\hookrightarrow$ start ESTa under Windows 
    \item C:\verb|\|ESTa\_Folder$>$ perl ESTa\_v0.1.beta.pl $<$enter$>$ $\hookrightarrow$ start ESTa under Linux/Mac
  \end{itemize}

=head2 Output-file

    \subsection{``EST-request\_(NCBI).txt''}
        \begin{itemize}
            \item The file ``EST-request\_(NCBI).txt'' contains the following structure:\\
                 ``Taxon'' \textsl{space} $=>$ \textsl{space} ``Number of EST entries found in the NCBI database \textsl{newline}''%
            \item ``EST-request\_(NCBI).txt'' lists all taxa with more than 1.000 entries in the NCBI-database
            \item The ``EST-request\_(NCBI).txt'' entries are listed alphabetically by taxon name%
        \end{itemize}
    \subsection{``New\_EST\_entries.txt''}
        \begin{itemize}
            \item The file ``New\_EST\_entries.txt'' contains the following structure:\\
		  ``Taxon'' \textsl{space} $=>$ \textsl{space} ``Number of EST entries found in the NCBI database \textsl{newline}''%
            \item The ``New\_EST\_entries.txt'' file lists all EST entries which which have changed since the last database update%
            \item The ``New\_EST\_entries.txt'' entries are listed alphabetically by taxon name
        \end{itemize}
=cut

use strict;
use LWP::UserAgent;
use HTTP::Response;
use URI::Heuristic;

my $rohe_url = ("http://www.ncbi.nlm.nih.gov/dbEST/dbEST_summary.html");
my $url      = URI::Heuristic::uf_urlstr($rohe_url);

$| = 1;
printf "%s =>\n\t", $url;

# User-Agent
my $ua = LWP::UserAgent->new();
   $ua->agent("Cassidy/v.6.66 Platinum");

# Referer
my   $response = $ua->get($url, Referer => "http://Holy.Alamo.com") ;
if ( $response->is_error() ){ printf " %s\n", $response->status_line }
else {
	
	my $inhalt = $response->content();
	my $bytes  = length $inhalt;
	my $anzahl = ($inhalt =~ tr/\n/\n/);
	
	printf "%s (%d Zeilen, %d Bytes)\n", $response->title() || "(kein Titel)", $anzahl, $bytes;
	my $inhalt_clean = substr($inhalt, index($inhalt, "<PLAINTEXT>"));
	my @taxa_est     = split "\n", $inhalt_clean;
	   @taxa_est     = sort @taxa_est ;
	
	my @input ;
	open IN, "<EST-request_(NCBI).txt";
	while (my $line =<IN>){ chomp $line and push @input, split " => ", $line }
	close IN ;
	
	my %est_entries_old = @input ;
	
	open  OUT, ">EST-request_(NCBI).txt";
	open  NEW, ">New_EST_entries.txt";
	print NEW  "New EST entries in NCBI:\n\n";
	
	my $counter = 1 ;
	for my $entry (1 .. @taxa_est){
		
		my @taxa             =  split " ", $taxa_est[$entry] ;
		my $N_est_entries    =  pop @taxa                    ; 
		   $N_est_entries    =~ s/,//g                       ;
		   $taxa_est[$entry] =  join  "_", @taxa             ; 
		   
		if ( $N_est_entries >= 1000 ){ print OUT "$taxa_est[$entry] => $N_est_entries\n" ;
			  
			if ($N_est_entries != $est_entries_old{$taxa_est[$entry]}){ print NEW "$counter\t$taxa_est[$entry] => $N_est_entries\n"; $counter++ }
		}
	}
	
	print NEW "\nNo further changes found.\n";
	close OUT and close NEW ;
}

print "\nDone!\n";
